package com.fzm.admin.listener;

import cn.fzm.chain.simplesdk.client.ParaChainClient;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.fzm.common.config.RabbitMQConfig;
import com.fzm.common.entity.CopyrightApply;
import com.fzm.common.entity.Nft;
import com.fzm.common.entity.dto.CopyrightQueryRequest;
import com.fzm.common.entity.dto.CopyrightQueryResponse;
import com.fzm.common.entity.vo.NftVo;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.properties.CopyrightProperties;
import com.fzm.common.service.CopyrightApplyService;
import com.fzm.common.service.NftService;
import com.fzm.common.utils.CopyrightSignUtil;
import com.fzm.common.utils.JsonUtil;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;

/**
 * @author tangtuo
 * @date 2021/12/23 14:56
 */
@Slf4j
@Component
public class CopyrightLister {

    @Resource
    private CopyrightProperties copyrightProperties;

    @Resource
    private RabbitTemplate rabbitTemplate;

    @Resource
    private CopyrightApplyService copyrightApplyService;

    @Resource
    private ParaChainClient paraChainClient;

    @Resource
    private NftService nftService;

    private static String abi = "[{\"inputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"constructor\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"ApprovalForAll\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"indexed\":false,\"internalType\":\"uint256[]\",\"name\":\"values\",\"type\":\"uint256[]\"}],\"name\":\"TransferBatch\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":true,\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"indexed\":true,\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"indexed\":false,\"internalType\":\"uint256\",\"name\":\"value\",\"type\":\"uint256\"}],\"name\":\"TransferSingle\",\"type\":\"event\"},{\"anonymous\":false,\"inputs\":[{\"indexed\":false,\"internalType\":\"string\",\"name\":\"value\",\"type\":\"string\"},{\"indexed\":true,\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"URI\",\"type\":\"event\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"}],\"name\":\"balanceOf\",\"outputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address[]\",\"name\":\"accounts\",\"type\":\"address[]\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"}],\"name\":\"balanceOfBatch\",\"outputs\":[{\"internalType\":\"uint256[]\",\"name\":\"\",\"type\":\"uint256[]\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"burnToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"tokenIds\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"}],\"name\":\"burnTokenBatch\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"}],\"name\":\"getTokenInfo\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"}],\"name\":\"getUserTokens\",\"outputs\":[{\"internalType\":\"uint256[]\",\"name\":\"\",\"type\":\"uint256[]\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"account\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"}],\"name\":\"isApprovedForAll\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"mintToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"tokenIds\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"}],\"name\":\"mintTokenBatch\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256[]\",\"name\":\"ids\",\"type\":\"uint256[]\"},{\"internalType\":\"uint256[]\",\"name\":\"amounts\",\"type\":\"uint256[]\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeBatchTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"from\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"id\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"},{\"internalType\":\"bytes\",\"name\":\"data\",\"type\":\"bytes\"}],\"name\":\"safeTransferFrom\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"operator\",\"type\":\"address\"},{\"internalType\":\"bool\",\"name\":\"approved\",\"type\":\"bool\"}],\"name\":\"setApprovalForAll\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"string\",\"name\":\"tokenInfo\",\"type\":\"string\"}],\"name\":\"setTokenInfo\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"bytes4\",\"name\":\"interfaceId\",\"type\":\"bytes4\"}],\"name\":\"supportsInterface\",\"outputs\":[{\"internalType\":\"bool\",\"name\":\"\",\"type\":\"bool\"}],\"stateMutability\":\"view\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"address\",\"name\":\"owner\",\"type\":\"address\"},{\"internalType\":\"address\",\"name\":\"to\",\"type\":\"address\"},{\"internalType\":\"uint256\",\"name\":\"tokenId\",\"type\":\"uint256\"},{\"internalType\":\"uint256\",\"name\":\"amount\",\"type\":\"uint256\"}],\"name\":\"transferToken\",\"outputs\":[],\"stateMutability\":\"nonpayable\",\"type\":\"function\"},{\"inputs\":[{\"internalType\":\"uint256\",\"name\":\"\",\"type\":\"uint256\"}],\"name\":\"uri\",\"outputs\":[{\"internalType\":\"string\",\"name\":\"\",\"type\":\"string\"}],\"stateMutability\":\"view\",\"type\":\"function\"}]";

    @Value("${chain.para.cAddr}")
    private String contractAddr;

    /**
     * 主动查询当前流水号对应的版权审批结果
     *
     * @param serial_code
     */
    @RabbitListener(queues = RabbitMQConfig.COPYRIGHT_QUEUE)
    public void listerCopyrightApply(String serial_code, Message message, Channel channel) throws IOException {
        log.info("监听者接收消息, 流水号为: {}", serial_code);
        long deliveryTag = message.getMessageProperties().getDeliveryTag();
        CopyrightQueryRequest request = new CopyrightQueryRequest(serial_code, copyrightProperties.getAppId());
        TreeMap<String, String> map = this.beanToMap(request, copyrightProperties.getAppSecret());
        String sign = CopyrightSignUtil.generateSign(map);
        request.setSign(sign);
        String json = JSONUtil.toJsonStr(request);
        log.info("查询审核结果, 当前流水号为:{} , 请求参数为 : {}", serial_code, json);
        String response = HttpUtil.post(copyrightProperties.getQueryUrl(), json, 10000);
        log.info("查询审核结果接口响应内容: {}", response);
        if (StringUtils.isBlank(response)) {
            // 接口无响应, 把消息丢进延时队列, 稍后重试
            rabbitTemplate.convertAndSend(RabbitMQConfig.COPYRIGHT_DIRECT, "copyright.apply", serial_code);
            channel.basicAck(deliveryTag, false);
            log.error("版权局查询存证接口无响应");
            return;
        }
        CopyrightQueryResponse copyrightResponse = JsonUtil.toObject(response, CopyrightQueryResponse.class);
        if (copyrightResponse == null) {
            // 接口无响应,手动签收, 把消息丢进延时队列, 稍后重试
            rabbitTemplate.convertAndSend(RabbitMQConfig.COPYRIGHT_DIRECT, "copyright.apply", serial_code);
            channel.basicAck(deliveryTag, false);
            log.error("版权局查询存证接口无响应");
            return;
        }
        if (!copyrightResponse.isSuccess()) {
            // 接口请求失败,手动签收, 把消息丢进延时队列, 稍后重试
            rabbitTemplate.convertAndSend(RabbitMQConfig.COPYRIGHT_DIRECT, "copyright.apply", serial_code);
            channel.basicAck(deliveryTag, false);
            log.error("接口响应失败, 失败原因:{}", copyrightResponse.getMessage());
            return;
        }
        try {
            //审核状态（0：未提交，1：待审核，2：同意，3：驳回，4：已提交，5：待终审，6：审核通过）
            if (!copyrightResponse.getAudit_status().equals(3) && !copyrightResponse.getAudit_status().equals(6)) {
                // 审核中, 需要把当前流水号丢进死信队列, 6小时后再去查询
                rabbitTemplate.convertAndSend(RabbitMQConfig.COPYRIGHT_DIRECT, "copyright.apply", serial_code);
            } else {
                CopyrightApply copyrightApply = copyrightApplyService.getBySerialNum(serial_code);
                if (copyrightResponse.getAudit_status().equals(6)) {
                    // 审核成功
                    String nftHash = copyrightApply.getNftHash();
                    copyrightApply.setEvidenceDate(DateUtil.parse(copyrightResponse.getApply_time(), "yyyy-MM-dd"));
                    copyrightApply.setRegisterCode(copyrightResponse.getCertificate().getRegister_code());
                    copyrightApply.setRegisterState(CopyrightApplyState.SUCCEEDED.getCode());
                    Nft nft = nftService.getByNftHash(nftHash);
                    String tokenInfo = paraChainClient.evmGetTokenInfo(abi, contractAddr, nft.getTokenId(), nft.getPublishAddress());
                    NftVo detail = nftService.getDetail(nftHash);
                    String newTokenInfo = appendTokenInfo(tokenInfo, detail.getCopyright());
                    log.info("追加tokenInfo, 新生成后的tokenInfo为: {}", newTokenInfo);
                    String hash = paraChainClient.evmSetTokenInfo(abi, contractAddr, 0, nft.getTokenId(), newTokenInfo, nft.getPublishAddress(), null, true);
                    if (StringUtils.isBlank(hash)) {
                        log.error("nft追加tokenInfo失败,当前流水号为: 【{}】", serial_code);
                        rabbitTemplate.convertAndSend(RabbitMQConfig.COPYRIGHT_DIRECT, "copyright.apply", serial_code);
                        return;
                    }
                    String realTxHash = paraChainClient.getRealTxHashFromGrp(hash);
                    copyrightApply.setEvidenceHash(realTxHash);
                } else {
                    // 审核失败
                    copyrightApply.setRegisterState(CopyrightApplyState.FAILED.getCode());
                }
                copyrightApplyService.updateById(copyrightApply);
            }
        } catch (Exception e) {
            // 如果上链失败或者更新状态失败, 那么就把消息重新丢入延时队列,稍后重试
            rabbitTemplate.convertAndSend(RabbitMQConfig.COPYRIGHT_DIRECT, "copyright.apply", serial_code);
            log.error(e.getMessage(), e);
        } finally {
            channel.basicAck(deliveryTag, false);
        }

    }

    private TreeMap<String, String> beanToMap(CopyrightQueryRequest request, String app_secret) {
        TreeMap<String, String> map = JSONUtil.toBean(JSONUtil.toJsonStr(request), new TypeReference<TreeMap<String, String>>() {
        }, true);
        map.put("app_secret", app_secret);
        return map;
    }

    private String appendTokenInfo(String tokenInfo, NftVo.Copyright copyright) {
        String[] split = tokenInfo.split("&");
        TreeMap<String, String> map = new TreeMap<>();
        for (String s : split) {
            String[] arr = s.split("=");
            String key = arr[0];
            String value;
            if (arr.length < 2) {
                value = "";
            } else {
                value = arr[1];
            }
            map.put(key, value);
        }
        map.put("evidencer", copyright.getEvidencer());
        map.put("opusName", copyright.getOpusName());
        map.put("opusType", copyright.getOpusType());
        map.put("author", copyright.getAuthor());
        map.put("copyrightOwner", copyright.getCopyrightOwner());
        map.put("registerCode", copyright.getRegisterCode());
        StringBuilder stringBuilder = new StringBuilder();
        for (Map.Entry<String, String> entry : map.entrySet()) {
            String value = entry.getValue();
            String key = entry.getKey();
            stringBuilder.append(key).append("=").append(value).append("&");
        }
        return stringBuilder.deleteCharAt(stringBuilder.length() - 1).toString();
    }

    public static void main(String[] args) {
//        String param = " {\"charset\":\"utf-8\",\"biz_content\":\"{\\\"serial_code\\\":\\\"21123030879\\\"}\",\"sign\":\"db432c9e6bc9d16a1e051f093d58ee33\",\"version\":\"1.0\",\"app_id\":\"289391457858\",\"sign_type\":\"md5\",\"timestamp\":\"2021-12-30 16:00:21\"}";
//        String response = HttpUtil.post("https://p.yareiot.com/ccct/api/copyright/query", param, 10000);
//        System.out.println(response);

        String str = "{\"success\":true,\"message\":\"查询成功\",\"code\":200,\"timestamp\":\"2021-12-31 17:26:44\",\"remark\":null,\"certificate\":null,\"sign\":\"eed26cc1a9d09cabe5ef1c984c8365d1\",\"copyright_detail\":{\"nature_code\":1,\"nature_desc\":\"\",\"nature_name\":null,\"end_time\":\"2021-12-16\",\"end_address\":\"上海市上海市\",\"works_desc\":\"哈哈哈哈哈哈\",\"works_create_desc\":\"哈哈哈哈哈哈哈\",\"publish_status\":1,\"publish_time\":null,\"publish_address\":null,\"agency_photo\":null,\"works_name\":\"依然范特西\",\"works_type_code\":3,\"works_type_name\":null,\"works_type_desc\":\"\",\"rights_affilia_code\":1,\"rights_affilia_name\":null,\"affilia_photo\":\"\",\"gain_code\":null,\"gain_name\":null,\"gain_desc\":null,\"geren_uname\":null,\"geren_uxname\":null,\"geren_uxname_type\":null,\"geren_uxname_type_name\":null,\"gain_photo\":\"\",\"author_photo\":\"\",\"entrust_photo\":null,\"rights_desc\":\"发版权、署名权、修改权、保护作品完整权、复制权、发行权、出租权、展览权、表演权、放映权、广播权、信息网络传播权、摄制权、改编权、翻译权、汇编权、其他\",\"other_photo\":\"\",\"rights_guarantee_photo\":\"https://test-nft.oss-cn-hangzhou.aliyuncs.com/20211230/9d623f3c8c94458cad029b0881238231/HupuBBS_211230152310-1242822577.png\",\"is_grade\":0,\"cooperator\":null,\"works_files\":[{\"file_name\":\"shareSongImage_temp_1630212140166.jpg\",\"file\":\"https://test-nft.oss-cn-hangzhou.aliyuncs.com/20211230/fb8e84e816e346d2b41e488224b292d0/shareSongImage_temp_1630212140166.jpg\",\"file_format\":\"jpg\",\"file_size\":\"19.21KB\",\"file_token\":\"2c06dde6f97213c133380199fec3eb33\",\"keywords\":null}],\"owner_list\":[{\"owner_name\":\"金庸\",\"owner_type_code\":2,\"document_type_code\":2,\"document_code\":\"34783huirh43i4834\",\"other_document_code\":null,\"document_image\":null,\"other_start_date\":null,\"other_end_date\":\"2021-12-22\",\"is_forever\":1,\"valid_start_date\":null,\"valid_end_date\":null,\"handheld_photo\":\"\",\"front_photo\":\"\",\"back_photo\":\"\",\"company_photo\":\"https://test-nft.oss-cn-hangzhou.aliyuncs.com/20211224/15489ec28bf241a6bae28f7f35cffb1d/微信图片_202102.jpg\",\"prove_photo\":\"https://test-nft.oss-cn-hangzhou.aliyuncs.com/20211224/cfc47db66eac46559ddf3c6583006d34/微信图片_20210802152956.jpg\",\"province\":\"天津市\",\"province_id\":\"120000\",\"city\":\"天津市\",\"city_id\":\"120100\",\"address\":\"天津市天津市\",\"owner_sign_name\":\"\",\"owner_type_name\":null,\"document_type_name\":null},{\"owner_name\":\"古龙\",\"owner_type_code\":1,\"document_type_code\":1,\"document_code\":\"45164646464694946\",\"other_document_code\":null,\"document_image\":null,\"other_start_date\":null,\"other_end_date\":\"2021-12-21\",\"is_forever\":1,\"valid_start_date\":null,\"valid_end_date\":null,\"handheld_photo\":\"https://test-nft.oss-cn-hangzhou.aliyuncs.com/20211230/4fb6881790c74d9c8de13581b0c46047/timg_image&quality=80&size=b9999_10000&sec=1588500161897&di=fcfcda647d253ea2fb5b858719070998&imgtype=0&src=http%3A%2F%2Fimg.jpg\",\"front_photo\":\"https://test-nft.oss-cn-hangzhou.aliyuncs.com/20211230/8d627d66ad4743b698b7a11df312f8cc/-624ec7938a1e0c8a.png\",\"back_photo\":\"https://test-nft.oss-cn-hangzhou.aliyuncs.com/20211230/1b4a307e7cb2411da809686117196ae6/-624ec7938a1e0c8a.png\",\"company_photo\":\"\",\"prove_photo\":\"\",\"province\":\"天津市\",\"province_id\":\"120000\",\"city\":\"天津市\",\"city_id\":\"120100\",\"address\":\"天津市天津市\",\"owner_sign_name\":\"\",\"owner_type_name\":null,\"document_type_name\":null}]},\"audit_status\":5,\"apply_time\":\"2021-12-30\"}";
//        CopyrightQueryResponse copyrightResponse = JSONUtil.toBean(str, new TypeReference<CopyrightQueryResponse>() {
//        },true);
        CopyrightQueryResponse copyrightQueryResponse = JsonUtil.toObject(str, CopyrightQueryResponse.class);
        //CopyrightQueryResponse copyrightResponse = JSONObject.parseObject(str, CopyrightQueryResponse.class);
        System.out.println(copyrightQueryResponse);
    }


}
