package com.fzm.common.config;

import org.springframework.amqp.core.*;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author tangtuo
 * @date 2021/12/23 13:25
 */
@Configuration
public class RabbitMQConfig {

    public static final String COPYRIGHT_DIRECT = "copyright.direct";

    public static final String DEAD_LETTER_DIRECT = "dl.direct";

    public static final String COPYRIGHT_QUEUE = "copyright.queue";

    public static final String DEAD_LETTER_QUEUE = "dl.queue";


    /**
     * 定义直连交换机copyright.direct
     *
     * @return
     */
    @Bean
    public DirectExchange copyrightDirect() {
        return new DirectExchange(COPYRIGHT_DIRECT, true, false, null);
    }

    /**
     * 死信交换机
     *
     * @return
     */
    @Bean
    public DirectExchange dlDirect() {
        return new DirectExchange(DEAD_LETTER_DIRECT, true, false, null);
    }

    /**
     * 存放死信的队列,用户监听这个队列
     *
     * @return
     */
    @Bean
    public Queue copyrightQueue() {
        return new Queue(COPYRIGHT_QUEUE, true);
    }


    /**
     * 定义死信队列
     *
     * @return
     */
    @Bean
    public Queue dlQueue() {
        return QueueBuilder.durable(DEAD_LETTER_QUEUE)
                .ttl(1000 * 60 * 60 * 6)
                .deadLetterExchange(DEAD_LETTER_DIRECT)
                .deadLetterRoutingKey("copyright.notify")
                .build();
    }

    @Bean
    public Binding dlBinding() {
        return BindingBuilder.bind(dlQueue()).to(copyrightDirect()).with("copyright.apply");
    }

    @Bean
    public Binding copyrightBinding() {
        return BindingBuilder.bind(copyrightQueue()).to(dlDirect()).with("copyright.notify");
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }
}
